/*
 * Copyright (C) 2011 Nathan Phillip Brink <ohnobinki@ohnopublishing.net>
 *
 * This file is a part of difffilter.
 *
 * difffilter is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * difffilter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with difffilter.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "pattern.h"

#include <stdio.h>
/* malloc() and free() */
#include <stdlib.h>

/* implementation of pattern.h */

difffilter_pattern_t difffilter_pattern_new(const char *pattern, short accept, short extended)
{
  difffilter_pattern_t new_diff_pattern;

  int ret;
  int regcflags;

  new_diff_pattern = malloc(sizeof(struct difffilter_pattern));
  if(!new_diff_pattern)
    return NULL;

  regcflags = REG_NOSUB;
  if (extended)
    regcflags |= REG_EXTENDED;
  ret = regcomp(&new_diff_pattern->preg, pattern, regcflags);
  if(ret)
    {
      diff_print_regerror(pattern, (reg_errcode_t)ret, &new_diff_pattern->preg);
      regfree(&new_diff_pattern->preg);
      free(new_diff_pattern);
      return NULL;
    }

  new_diff_pattern->accept = accept;

  return new_diff_pattern;
}

void difffilter_pattern_free(difffilter_pattern_t difffilter_pattern)
{
  regfree(&difffilter_pattern->preg);
  free(difffilter_pattern);
}

int diff_print_regerror(const char *pattern, reg_errcode_t errcode, const regex_t *preg)
{
  size_t errstring_len;
  char *errstring;

  errstring_len = regerror(errcode, preg, NULL, 0);
  errstring = malloc(errstring_len);
  regerror(errcode, preg, errstring, errstring_len);

  fprintf(stderr, "Error parsing \"%s\": %s", pattern, errstring);
  free(errstring);

  return 0;
}
