/*
 * Copyright (C) 2011 Nathan Phillip Brink <ohnobinki@ohnopublishing.net>
 *
 * This file is a part of difffilter.
 *
 * difffilter is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * difffilter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with difffilter.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "block.h"

#include <stdio.h>
#include <string.h>
#include <unistd.h>

const char *diff_lines[] =
  {
    "@@ -1,2 +1,2 @@\n",
    "-a\n",
    "+b\n",
    " \n",
    (char *)NULL
  };

int main(int argc, char *argv[])
{
  diff_block_t diff_block;
  size_t counter;

  int tmp;
  size_t len;

  char *buf;
  char *buf_ptr;

  size_t orig_begin;
  size_t new_begin;

  char *orig;
  size_t orig_len;
  char *new;
  size_t new_len;

  diff_block = diff_block_init(1, 1);
  for(counter = 1; diff_lines[counter]; counter ++)
    {
      tmp = diff_block_interpret(diff_block, diff_lines[counter]);
      if(tmp)
	{
	  fprintf(stderr, "diff_block_interpret() returned %d\n", tmp);
	  return 1;
	}
    }

  diff_block_begin_get(diff_block, &orig_begin, &new_begin);
  if (orig_begin != 1 || new_begin != 1)
    {
      fprintf(stderr, "diff_block_begin_get() gave us these unexpected values: orig_begin = %zu, new_begin = %zu\n",
	      orig_begin, new_begin);
      return 1;
    }

  tmp = diff_block_reconstruct(diff_block, &orig, &orig_len, &new, &new_len);
  if (tmp)
    {
      fprintf(stderr, "diff_block_reconstruct() returned %d\n", tmp);
      return 1;
    }
  if (strcmp("a\n\n", orig))
    {
      fprintf(stderr, "diff_block_reconstruct() gave me the following for the origin instead of ``a\\n\\n'': ``%s''\n",
	      orig);
      return 1;
    }
  if (strcmp("b\n\n", new))
    {
      fprintf(stderr, "diff_block_reconstruct() gave me the following for the new instead of ``b\\n\\n'': ``%s''\n",
	      new);
      return 1;
    }

  len = diff_block_render(diff_block, &buf);
  buf_ptr = buf;

  for(counter = 0; diff_lines[counter]; counter ++)
    {
      if(strncmp(buf_ptr, diff_lines[counter], strlen(diff_lines[counter])))
	{
	  buf_ptr[strlen(diff_lines[counter])] = '\0';
	  fprintf(stderr, "     got: %s\nexpected: %s\n",
		  buf_ptr, diff_lines[counter]);
	  return 1;
	}
      buf_ptr += strlen(diff_lines[counter]);
    }

  if(buf_ptr != buf + len)
    {
      fprintf(stderr, "buf is %lu bytes instead of %lu bytes long\n",
	      (unsigned long)(buf_ptr - buf), (unsigned long)len);
      return 1;
    }

  diff_block_free(diff_block);

  return 0;
}
