/*
 * Copyright (C) 2011 Nathan Phillip Brink <ohnobinki@ohnopublishing.net>
 *
 * This file is a part of difffilter.
 *
 * difffilter is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * difffilter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with difffilter.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "block.h"
#include "diff.h"

struct diff_visit_blocks_func
{
  short noop;
};
/**
 * \brief Some function to mess with each block of the diff somehow.
 */
enum diff_visit_blocks_func_return diff_visit_blocks_func(diff_block_t block, void *data)
{
  struct diff_visit_blocks_func *blocks_func = data;

  if(blocks_func->noop)
    return DIFF_VISIT_BLOCKS_KEEP;

  /**
   * \todo mess with the block.
   */

  return DIFF_VISIT_BLOCKS_KEEP;
}

int main(int argc, char *argv[])
{
  diff_t diff;
  diff_block_t block;

  int diff_block_offsets[2][2] =
    {
      {27, 27},
      {44, 45}
    };
  const char *diff_block_lines[3][20] =
    {
      {
	" difffilter_CFLAGS = $(LIBLIST_CFLAGS) $(TRE_CFLAGS)\n",
	" difffilter_SOURCES = \\\n",
	" 	src/block.c src/block.h \\\n",
	"+	src/diff.c src/diff.h \\\n",
	" 	src/difffilter.c \\\n",
	" 	src/pattern.c src/pattern.h\n",
	" \n",
	NULL
      },
      {
	" \n",
	" difffilter.1: doc/difffilter.1.txt\n",
	" 	txt2man -P $(PACKAGE) -r '$(PACKAGE_STRING)' -s 1 -t difffilter $< > $@\n",
	"+\n",
	NULL
      },
      {
	NULL
      },
    };
  const char *correct =
"--- a/Makefile.am\n\
+++ b/Makefile.am\n\
@@ -27,6 +27,7 @@\n\
 difffilter_CFLAGS = $(LIBLIST_CFLAGS) $(TRE_CFLAGS)\n\
 difffilter_SOURCES = \\\n\
 \tsrc/block.c src/block.h \\\n\
+\tsrc/diff.c src/diff.h \\\n\
 \tsrc/difffilter.c \\\n\
 \tsrc/pattern.c src/pattern.h\n\
 \n\
@@ -44,3 +45,4 @@\n\
 \n\
 difffilter.1: doc/difffilter.1.txt\n\
 \ttxt2man -P $(PACKAGE) -r '$(PACKAGE_STRING)' -s 1 -t difffilter $< > $@\n\
+\n";

  size_t block_counter;
  size_t line_counter;

  struct diff_visit_blocks_func blocks_func_data;

  int tmp;

  size_t counter;
  size_t len;
  char *buf;

  diff = diff_init("a/Makefile.am", "b/Makefile.am");
  if(!diff)
    {
      fprintf(stderr, "Error allocating diff\n");
      return 1;
    }

  for(block_counter = 0; diff_block_lines[block_counter][0]; block_counter ++)
    {
      block = diff_block_init(diff_block_offsets[block_counter][0],
			      diff_block_offsets[block_counter][1]);
      if(!block)
	{
	  fprintf(stderr, "Error allocating diff_block\n");
	  return 1;
	}
    for(line_counter = 0; diff_block_lines[block_counter][line_counter]; line_counter ++)
      {
	tmp = diff_block_interpret(block, diff_block_lines[block_counter][line_counter]);
	if(tmp)
	  {
	    fprintf(stderr, "Error calling diff_block_interpret() on ``%s''\n",
		    diff_block_lines[block_counter][line_counter]);
	    return 1;
	  }
      }
    diff_add(diff, block);
    }

  blocks_func_data.noop = 1;
  tmp = diff_visit_blocks(diff, &diff_visit_blocks_func, &blocks_func_data);
  if(tmp)
    {
      fprintf(stderr, "error: diff_visit_blocks() returned %d\n",
	      tmp);
      return 1;
    }

  len = diff_render(diff, &buf);
  if(len == -1)
    {
      fprintf(stderr, "diff_render(): Error rendering diff\n");
      return 1;
    }

  if(strncmp(buf, correct, len))
    {
      fprintf(stderr, "Diff output is incorrect: got the following:\n");
      for(counter = 0; counter < len; counter ++)
	putchar(buf[counter]);
      fprintf(stderr, "expected:\n%s", correct);
      return 1;
    }

  free(buf);

  diff_free(diff);

  return 0;
}

