#!/bin/sh
#
# Copyright (C) 2011 Nathan Phillip Brink <ohnobinki@ohnopublishing.net>
#
# This file is a part of difffilter.
#
# difffilter is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# difffilter is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with difffilter.  If not, see <http://www.gnu.org/licenses/>.

diffsdir="${srcdir}"/tests/diffs
outdir="tests/outfiles"

. "${srcdir}"/tests/sh-common.sh

for prog in cmp; do
    check_program_available ${prog}
done

mkdir -p "${outdir}"

# Diffs which difffilter should process with no problem and spit out
# unchanged.
DIFFS="a malformed-cvs-keyword.patch"

outfile="${outdir}"/diffs.diff
for diff in ${DIFFS}; do
    DIFFFILTER_FLAGS=""

    if [ ${diff} = "malformed-cvs-keyword.patch" ]; then
	DIFFFILTER_FLAGS="-k Id"
    fi

    diff="${diffsdir}"/${diff}
    if ! ./difffilter ${DIFFFILTER_FLAGS} < "${diff}" > "${outfile}"; then
	local ret=$?
	echo "Failed: \`difffilter ${DIFFFILTER_FLAGS}' exited with exit code ${ret}." >&2
	exit 1
    fi

    # remember, difffilter isn't supposed to do anything with these files.
    if ! cmp "${diff}" "${outfile}"; then
	echo "Failed: difffilter modified the diff \`${diff}', resulting in \`${outfile}." >&2
	echo 
	echo ./difffilter ${DIFFFILTER_FLAGS}
	cat ${outfile}
	exit 1
    fi
done

rm "${outfile}"
